#!/bin/sh
### Settings
UPURL="ccr@low.fi:public_html/ggrtf"
SRCFILE="manual.sgml"
SRCTMP="manual.xml"
FOTMP="tmp.fo"
HTMLPATH="html"
HTMLFILE="$HTMLPATH/manual.html"
PDFFILE="manual.pdf"
TXTFILE="manual.txt"
PSFILE="manual.ps"
XSLTPARAMS="--stringparam html.stylesheet manual.css"

### Get paths, if available
echo "* Initialization"
CPWD=`pwd`
GS=`which gs`
CONVERT=`which convert`
PNMTOPS=`which pnmtops`
GIFTOPNM=`which giftopnm`
TIDY=`which tidy`
XSLTPROC=`which xsltproc`
XMLROFF=`which xmlroff`
CATALOGS1=`locate xhtml/chunk.xsl|head -1`
CATALOGS2=`locate xhtml/docbook.xsl|head -1`
CATALOGS3=`locate fo/docbook.xsl|head -1`
LINKS=`which links`
ELINKS=`which elinks`
LYNX=`which lynx`

### Convert images
echo "* Checking for image to EPS converting utilities ..."
if test -n "$PNMTOPS" && test -x "$PNMTOPS" && test -n "$GIFTOPNM" && test -x "$GIFTOPNM"; then
	echo "** NetPBM tools found"
	for i in *.gif; do
		TOUTFILE=`echo "$i" | sed "s/.gif/.eps/"`
		echo " - $i -> $TOUTFILE"
		$GIFTOPNM "$i" | $PNMTOPS -scale=0.5 -noturn > "$TOUTFILE"
	done
elif test -n "$CONVERT" && test -x "$CONVERT"; then
	echo "** ImageMagick found"
	for i in *.gif; do
		TOUTFILE=`echo "$i" | sed "s/.gif/.eps/"`
		echo " - $i -> $TOUTFILE"
		$CONVERT "$i" "$TOUTFILE"
	done
else
	echo "*** WARNING! No supported image conversion tools found, not converting images."
	echo "*** This may cause some document format conversions to fail or produce errors."
fi


### Check that we have xsltproc installed for DocBook XML processing
if test -n "$XSLTPROC" && test -x "$XSLTPROC"; then
	echo "*** xsltproc found, good... testing for DocBook XML stuff .."
	if test -n "$CATALOGS1"; then
		echo "  * $CATALOGS1 found."
	else
		echo "*** xhtml/chunk.xsl not found!"
		exit 1
	fi
	if test -n "$CATALOGS2"; then
		echo "  * $CATALOGS2 found."
	else
		echo "*** xhtml/docbook.xsl not found!"
		exit 1
	fi
	if test -n "$CATALOGS3"; then
		echo "  * $CATALOGS3 found."
	else
		echo "*** fo/docbook.xsl not found!"
		exit 1
	fi
else
	echo "*** ERROR! Could not find xsltproc! You need xsltproc and the"
	echo "*** DocBook XML suite (with XSL stylesheets) to be able to"
	echo "*** generate the documentation!"
	exit 1
fi


### Generate kludge-normalized XML from the DocBook SGML source
perl -w normalizeml.pl < "$SRCFILE" > "$SRCTMP"


### Generate HTML files
echo "* HTML"
if test -e "$HTMLPATH"; then
	rm -fr "$HTMLPATH"
fi
mkdir -p "$HTMLPATH"

if test -d "$HTMLPATH"; then
	cp *.css *.png *.gif "$HTMLPATH" && cd "$HTMLPATH" && $XSLTPROC $XSLTPARAMS "$CATALOGS1" "../$SRCTMP"
	cd "$CPWD"
	
	if $XSLTPROC $XSLTPARAMS "$CATALOGS2" "$SRCTMP" > "$HTMLFILE"; then
		echo "** Transform successful."
	else
		echo "*** ERROR! $XSLTPROC failed. Quitting on fatal error."
		echo "*** Check $SRCTMP for errors."
		exit 2
	fi

	echo "** Checking for HTML Tidy ..."
	if test -n "$TIDY" && test -x "$TIDY"; then
		echo "** Found, cleaning up the mess by DocBook .."
		if $TIDY -q -w 120 -utf8 -asxhtml -i -m $HTMLPATH/*.html "$HTMLFILE"; then
			echo "** Tidying process successful."
		fi
	else
		echo "*** WARNING! HTML tidy not found! To get better HTML output,"
		echo "*** please install HTML tidy (http://tidy.sourceforge.net/)"
	fi
fi


### Generate PostScript file
#rm -f "$PSFILE"
#if test -n "$XMLROFF" && test -x "$XMLROFF" && test -e "$SRCTMP"; then
#	echo "* PostScript"
#	$XSLTPROC "$CATALOGS3" "$SRCTMP" > "$FOTMP"
#	$XSLTPROC "$CATALOGS3" "$SRCTMP" > BASCE
#fi


### Generate PDF
if test -n "$GS" && test -x "$GS" && test -e "$PSFILE"; then
	echo "* PDF ..."
	rm -f "$PDFFILE"
	if $GS -dSAFER -dCompatibilityLevel=1.2 -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite \
	-sOutputFile=$PDFFILE -dCompatibilityLevel=1.2 -c .setpdfwrite -f manual.ps; then
		echo "*** PDF conversion successful."
	else
		echo "*** ERROR! $GS failed. Non-fatal error, continuing."
	fi
elif test -e "$PSFILE"; then
	echo "*** WARNING! GhostScript does not seem to be installed."
	echo "*** PDF version of the manual will not be generated."
else
	echo "*** PostScript file was not generated, unable to generate"
	echo "*** PDF version of the manual, skipping this part."
fi

### Generate ASCII text
if test -e "$HTMLFILE"; then
	echo "* Text"
	if test -n "$LINKS" && test -x "$LINKS"; then
		$LINKS -dump "$HTMLFILE" > "$TXTFILE"
	elif test -n "$ELINKS" && test -x "$ELINKS"; then
		$ELINKS -dump "$HTMLFILE" > "$TXTFILE"
	elif test -n "$LYNX" && test -x "$LYNX"; then
		echo "*** FIXME! lynx found, don't know how to transform."
	else
		echo "*** WARNING! No Links, ELinks or Lynx found. Not generating plaintext documentation."
	fi
fi

### Remove temporary files
rm -f "$SRCTMP" "$FOTMP"

## Upload
if test "x$1" = "x--upload"; then
	scp $HTMLPATH/* manual.css "$UPURL/html/"
	scp "$TXTFILE" "$PSFILE" "$PDFFILE" "$UPURL"
fi
